@@ -157,6 +157,9 @@ class AccountEntry(models.Model):
     amount = models.DecimalField(
         verbose_name=_("amount"), max_digits=10, decimal_places=2, blank=True, default=None, null=True, db_index=True
     )
+    cached_balance = models.DecimalField(
+        verbose_name=_("balance"), max_digits=10, decimal_places=2, blank=True, default=None, null=True
+    )
     source_file = models.ForeignKey(
         AccountEntrySourceFile,
         verbose_name=_("account entry source file"),
@@ -233,13 +236,17 @@ class AccountEntry(models.Model):
     def balance(self) -> Decimal:
         """
         Returns account balance after this entry.
+        Entries are primarily sorted by timestamp followed by id.
         :return: Decimal
         """
-        return sum_queryset(
-            AccountEntry.objects.filter(account=self.account, timestamp__lte=self.timestamp).exclude(
-                timestamp=self.timestamp, id__gt=self.id
+        if self.cached_balance is None:
+            self.cached_balance = sum_queryset(
+                AccountEntry.objects.filter(account=self.account, timestamp__lte=self.timestamp).exclude(
+                    timestamp=self.timestamp, id__gt=self.id
+                )
             )
-        )
+            self.save(update_fields=["cached_balance"])
+        return self.cached_balance
 
     balance.fget.short_description = _("balance")  # type: ignore  # pytype: disable=attribute-error
 
@@ -279,6 +286,7 @@ class Account(models.Model):
     last_modified = models.DateTimeField(
         verbose_name=_("last modified"), auto_now=True, db_index=True, editable=False, blank=True
     )
+    notes = models.TextField(_("notes"), blank=True, default="")
 
     class Meta:
         verbose_name = _("account")
@@ -301,7 +309,11 @@ class Account(models.Model):
 
     @property
     def balance(self) -> Decimal:
-        return sum_queryset(self.accountentry_set.all())
+        e = self.accountentry_set.all().order_by("-timestamp", "-id").first()
+        if e is None:
+            return Decimal("0.00")
+        assert isinstance(e, AccountEntry)
+        return e.balance
 
     balance.fget.short_description = _("balance")  # type: ignore  # pytype: disable=attribute-error
 
@@ -333,6 +345,16 @@ class Account(models.Model):
             and AccountEntry.objects.filter(parent=e).count() == 0
         )
 
+    def calculate_balances(self):
+        bal = Decimal("0.00")
+        for e in AccountEntry.objects.all().filter(account=self).order_by("timestamp", "id"):
+            assert isinstance(e, AccountEntry)
+            if e.amount is not None:
+                bal += e.amount
+            if e.cached_balance is None or e.cached_balance != bal:
+                e.cached_balance = bal
+                e.save(update_fields=["cached_balance"])
+
 
 class InvoiceManager(models.Manager):
     @transaction.atomic
